#include <windows.h>

typedef struct {
    HICON icono;
    POINT coordenada;
} Objeto;

/* Declaracin del procedimiento de ventana */
LRESULT CALLBACK WindowProcedure(HWND, UINT, WPARAM, LPARAM);

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */

    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (hThisInstance, "icono");
    wincl.hIconSm = LoadIcon (hThisInstance, "peque");
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = NULL;
    wincl.cbClsExtra = 0;    /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;    /* clase o la ventana */
    /* Usar el color de fondo por defecto para es escritorio */
    wincl.hbrBackground =  GetSysColorBrush(COLOR_BACKGROUND); //GetStockObject(WHITE_BRUSH);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 035",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           500,
           450,
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        TranslateMessage(&mensaje);
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
    }

    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/* Esta funcin es llamada por la funcin del API DispatchMessage() */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    HDC hdc;
    PAINTSTRUCT ps;
    static HINSTANCE hInstance;
    static Objeto objeto[5];
    static POINTS punto;
    POINT lpunto;
    static int capturado;
    int i;
    RECT re;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_CREATE:
           hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
           objeto[0].icono = LoadIcon(hInstance, "ufo");
           objeto[0].coordenada.x = 10;
           objeto[0].coordenada.y = 10;
           objeto[1].icono = LoadIcon(hInstance, "libro");
           objeto[1].coordenada.x = 10;
           objeto[1].coordenada.y = 50;
           objeto[2].icono = LoadIcon(hInstance, "mundo");
           objeto[2].coordenada.x = 10;
           objeto[2].coordenada.y = 90;
           objeto[3].icono = LoadIcon(hInstance, "hamburguesa");
           objeto[3].coordenada.x = 50;
           objeto[3].coordenada.y = 10;
           objeto[4].icono = LoadIcon(hInstance, "smile");
           objeto[4].coordenada.x = 50;
           objeto[4].coordenada.y = 50;
           capturado = -1;
           break;
        case WM_MOUSEMOVE:
           punto = MAKEPOINTS(lParam);
           if(capturado != -1) {
              hdc = GetDC(hwnd);
              //drag
              // Borrar en posicin actual:
              SetRect(&re,
                 objeto[capturado].coordenada.x,
                 objeto[capturado].coordenada.y,
                 objeto[capturado].coordenada.x+32,
                 objeto[capturado].coordenada.y+32);
              FillRect(hdc, &re, GetSysColorBrush(COLOR_BACKGROUND));
              // Actualizar coordenadas:
              POINTSTOPOINT(objeto[capturado].coordenada, punto);
              // Pintar en nueva posicin:
              DrawIcon(hdc,
                 objeto[capturado].coordenada.x,
                 objeto[capturado].coordenada.y,
                 objeto[capturado].icono);
              ReleaseDC(hwnd, hdc);
              InvalidateRect(hwnd, &re, FALSE);
           }
           break;
        case WM_LBUTTONDOWN:
           punto = MAKEPOINTS(lParam);
           for(i = 0; capturado == -1 && i < 5; i++) {
              SetRect(&re,
                objeto[i].coordenada.x,
                objeto[i].coordenada.y,
                objeto[i].coordenada.x+32,
                objeto[i].coordenada.y+32);
              POINTSTOPOINT(lpunto, punto);
              if(PtInRect(&re, lpunto)) {
                 capturado = i;
                 ClientToScreen(hwnd, &objeto[i].coordenada);
                 SetCursorPos(objeto[i].coordenada.x,
                    objeto[i].coordenada.y);
                 SetCapture(hwnd);
                 ShowCursor(FALSE);
              }
           }
           break;
        case WM_LBUTTONUP:
           if(capturado != -1) {
              capturado = -1;
              InvalidateRect(hwnd, NULL, FALSE);
              ReleaseCapture();
              ShowCursor(TRUE);
           }
           break;
        case WM_PAINT:
           hdc = BeginPaint(hwnd, &ps);
           for(i = 0; i < 5; i++) {
              DrawIcon(hdc,
                 objeto[i].coordenada.x,
                 objeto[i].coordenada.y,
                 objeto[i].icono);
           }
           EndPaint(hwnd, &ps);
           break;
        case WM_DESTROY:
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

